@@ -86,11 +86,7 @@ setReplaceMethod("rhs", signature(x = "rules"),
 
 ## get the union of rhs and lhs
 setMethod("items", signature(x = "rules"),
-    function(x) {
-        x@lhs@data <- .Call("R_or_ngCMatrix", x@lhs@data, x@rhs@data, 
-		PACKAGE="arules")
-        x@lhs
-    }
+    function(x) itemUnion(x@lhs, x@rhs)
 )
 
 # get the generating itemsets
